#include <libguile.h>
#include <command.hh>
#include <command_io.hh>
#include <iostream>

class mapper
{
	SCM mdata;

public:
	struct missing {};

	inline mapper(const std::string& filename);

	inline command
	operator[](const std::string&) const;

	inline command
	map(const std::string&) const;

	command&
	map(const std::string&, command&) const;
};

SCM load_mapper(const std::string& filename);

inline command
mapper::operator[](const std::string& viewid) const
{
	return map(viewid);
}

inline
mapper::mapper(const std::string& filename): mdata(SCM_BOOL_F)
{
	mdata = load_mapper(filename);
}

inline command
mapper::map(const std::string& viewid) const
{
	command cmd;
	return map(viewid, cmd);
}

command&
mapper::map(const std::string& viewid, command& cmd) const
{
	SCM result = scm_call_2(scm_variable_ref(scm_c_lookup("map-view")),
			mdata,
			scm_mem2string(viewid.c_str(), viewid.size()));
	SCM filesym = scm_str2symbol("file");
	SCM viewsym = scm_str2symbol("view");
	SCM printer = scm_variable_ref(scm_c_lookup("display"));
	if(!SCM_CONSP(result)) throw missing();
	if(SCM_NFALSEP(scm_eq_p(SCM_CAR(result), filesym)))
	{
		cmd("action", "file");
		if(SCM_NFALSEP(scm_null_p(SCM_CDR(result)))) throw missing();
		SCM name = scm_object_to_string(
				SCM_CADR(result), printer);
		cmd("name", std::string(SCM_STRING_CHARS(name),
					SCM_STRING_LENGTH(name)));
	}
	else if(SCM_NFALSEP(scm_eq_p(SCM_CAR(result), viewsym)))
	{
		cmd("action", "view");
		if(SCM_NFALSEP(scm_null_p(SCM_CDR(result)))) throw missing();
		SCM id = scm_object_to_string(
				SCM_CADR(result), printer);
		cmd("id", std::string(SCM_STRING_CHARS(id),
					SCM_STRING_LENGTH(id)));
	}
	else throw missing();
	cmd.name("/ctrl/look");
	result = SCM_CDDR(result);
	while(!SCM_NFALSEP(scm_null_p(result)))
	{
		SCM name = scm_object_to_string(
				SCM_CAR(result), printer);
		if(SCM_NFALSEP(scm_null_p(SCM_CDR(result)))) break;
		SCM value = scm_object_to_string(
				SCM_CADR(result), printer);
		cmd(std::string(SCM_STRING_CHARS(name),
					SCM_STRING_LENGTH(name)),
			std::string(SCM_STRING_CHARS(value),
					SCM_STRING_LENGTH(value)));
		result = SCM_CDDR(result);
	}
	return cmd;
}

SCM load_mapper(const std::string& mapfile)
{
	return scm_call_1(scm_variable_ref(scm_c_lookup("load-mapper")),
			scm_mem2string(mapfile.c_str(), mapfile.size()));
}

void test_mapper(const std::string& id)
{
	mapper m("src/prg/map.scm");
	try
	{
		std::cout << m[id] << std::endl;
	}
	catch(mapper::missing&)
	{
		std::cout << "no such mapping" << std::endl;
	}
}
