(use-modules (ice-9 regex) (srfi srfi-1))

(define *sandbox* #f)

(defmacro later x `(lambda () ,@x))

(define (sandbox)
  (if *sandbox* *sandbox*
      (begin (set! *sandbox* (make-module))
	     (module-define! *sandbox* 'lambda lambda)
	     (module-define! *sandbox* 'later later)
	     (module-define! *sandbox* 'make-regexp make-regexp)
	     *sandbox*)))

(define (cond-cons x y) (and x (cons x y)))

(define (load-mapper filename)
  (with-input-from-file filename
    (lambda () 
      (let loop ((e (read)))
	(if (eof-object? e) '()
	    (fold-right
	      (lambda (x y) (or (cond-cons (parse-mapper-item x) y) y))
	      (loop (read))
	      (perform-unquote e)))))))

(define (parse-mapper-item item)
  (and (pair? item) (cond-cons (convert-key (car item)) (cdr item))))

(define (convert-key key)
  (cond ((string? key) (lambda (x) (string=? key x)))
	((regexp? key) (lambda (x) (regexp-exec key x)))
	((null? key) (lambda (x) #t))
	((pair? key) (apply disjoin (map convert-key key)))
	((symbol? key) (convert-key (symbol->string key)))
	((procedure? key) key)
	(else #f)))

(define (map-view mapper id)
  (if (null? mapper) #f
      (or (test-and-eval ((caar mapper) id) (cdar mapper))
	  (map-view (cdr mapper) id))))

(define (test-and-eval ctxt answer)
  (if (not ctxt) #f
      (begin
	(if (vector? ctxt)
	    (module-define! (sandbox) 'match
			    (lambda args
			      (apply regexp-substitute #f ctxt args)))
	    (module-remove! (sandbox) 'match))
	(fold-right
	  (lambda (x y) (cons (if (procedure? x) (x) x) y)) '() answer))))

(define (perform-unquote e)
  (if (pair? e)
      (case (car e)
	((unquote) (list (eval (cadr e) (sandbox))))
	((unquote-splicing) (eval (cadr e) (sandbox)))
	((include) (load-mapper (cadr e)))
	(else (list (fold-right append '() (map perform-unquote e)))))
      (list e)))

(define (disjoin . funcs)
  (lambda args (any? (lambda (f) (apply f args)) funcs)))

(define (any? pred lst)
  (if (null? lst) #f
      (or (pred (car lst))
	  (any? pred (cdr lst)))))
